#include <stdio.h>
#include <assert.h>
#include "include/actor.h"
#include "include/mainlib.h"
#include "include/behaviour.h"	/* for B_YRespawn and B_Player */

extern const int maxActors;	/* Maximum actors in game at one time */
extern Actor *ActorList;	/* ptr to memory for storing the Actors */

extern int scrH; /* Screen height */
extern int scrW; /* Screen width  */

/* registerActor(): Add an Actor to the ActorList */
/* Returns a pointer to the Actor, or NULL if the
** coordinates are out of bounds  */
Actor* registerActor(int ypos, int xpos, int mode, void(*behaviour)(Actor*)){
	int i=0;

	/* Take care of bad values */
	if(behaviour == NULL){
		printf("registerActor called with NULL behaviour!\n");
		assert(0);
	}
	if(ypos >= scrH || ypos < 0 || xpos >= scrW || xpos < 0){
		return NULL;	/* Return NULL when out of bounds */
	}

	/* Find a place to put the new Actor */
	while(i<maxActors && ActorList[i].behaviour != NULL) i++;
	if(i>=maxActors){
 		/* Leave a spot open for a respawner */
		if(i != maxActors-1 &&
			behaviour != B_YRespawn && behaviour != B_Player)
			/* If there is no room, do nothing */
			return NULL;
		/* Given the current logic and data structures,
		** ActorList[] cannot be realloced to a larger size
		** because registerActor() is called from the middle of
		** behavior functions that use pointers to
		** data within the ActorList array. The array would
		** be moved while the pointers would still point to
		** the old memory location, crashing the program.
		*/
	}

	/* Set the new Actor's settings */
	ActorList[i].xpos = xpos; 
	ActorList[i].ypos = ypos; 
	ActorList[i].mode = mode; 
	ActorList[i].behaviour = behaviour; 
	/* Debugging:
	if(behaviour == B_Rock && mode < 1) assert(0);
	if(behaviour == B_Bonus && (mode<1||mode>4)) assert(0);
	*/
	return &(ActorList[i]);
}


/* fixActorList(): Remove dead Actors from the list */
void fixActorList(){
	int i=0,		/* Count up from the beginning */
	    j=maxActors-1;	/* Count down from the end */
	while (i<j){
		if(ActorList[i].behaviour == NULL
			&& ActorList[j].behaviour != NULL){
			memcpy(&(ActorList[i]),&(ActorList[j]),sizeof(Actor));
			ActorList[j].behaviour = NULL;
			--j; ++i;
		}
		if(ActorList[j].behaviour == NULL) j--;
		if(ActorList[i].behaviour != NULL) i++;
		/* The counter at the end waits on a live
		actor for the other counter to find an empty
		space to move it into. Similarly, the counter
		at the beginning waits on an empty space for
		the counter at the end to find an actor to
		move into the space. */
	}
}

