#include <stdlib.h>	/* for NULL */
#include <assert.h>
#include <curses.h>	/* for KEY_UP, etc */
#include "include/behaviour.h"
#include "include/actor.h"

extern long int random (void); /* Compiler doesn't pick it up from stdlib.h */

extern int playerCount;
extern int scrW;
extern int scrH;
extern int score;
extern long frame;	/* Frame number. */
extern int *data;	/* Data buffer of last complete frame */
extern int *xdata;	/* Data buffer of current frame */
extern long input;
extern int maxActors;
extern Actor * ActorList;

/*** Behaviour modes ***/

/* Behaviour for an explosion */
void B_Explode(Actor* self){
	xdata[self->xpos+self->ypos*scrW] = '*';	/* Go boom */
	if(--self->mode < 0){
		self->behaviour = NULL;			/* Go away */
	}
}


/* Behaviour function for player-controlled object */
void B_Player(Actor* self){

switch(input) {
	/* Fire commands */
	case ' ':	/* Fall through */
	case 'z': registerActor(self->ypos, (self->xpos+1), 0, &B_Yourshot);
		break;

	/* Movement commands */
	case KEY_DOWN:	/* Fall through */
	case 'a':	/* Fall through */
	case 'J':	/* Fall through */
	case '2':	/* Fall through */
	case 'j': if(self->ypos < (scrH-1)) self->ypos++;
		break;

	case KEY_RIGHT:	/* Fall through */
	case 'p':	/* Fall through */
	case 'L':	/* Fall through */
	case '6':	/* Fall through */
	case 'l': if(self->xpos<(scrW-1)) self->xpos++;
		break;

	case KEY_LEFT:	/* Fall through */
	case 'o':	/* Fall through */
	case 'H':	/* Fall through */
	case '4':	/* Fall through */
	case 'h': if(self->xpos>0) self->xpos--;
		break;

	case KEY_UP:	/* Fall through */
	case 'q':	/* Fall through */
	case 'K':	/* Fall through */
	case '8':	/* Fall through */
	case 'k': if(self->ypos>0) self->ypos--;
		break;

	default: break;
	}

	/* Die upon touching anything *
	if(xdata[self->xpos+self->ypos*scrW] != ' '){
		self->behaviour=B_Explode;
		self->behaviour(self);
	} else {
		xdata[self->xpos+self->ypos*scrW] = '>';
		playerCount++;
	}*/
	FindColl(self);
	if(self->behaviour != NULL){
		playerCount++;
		xdata[self->xpos+self->ypos*scrW] = '>';
	}
}

/* Behaviour for player's shot */
/* Modes: 0 -- Just spawned, don't move
**        1 -- Okay, you can move now  */
void B_Yourshot(Actor* self){

	if(self->mode==0) self->mode = 1;	/* don't move, just spawned */
	else if((++(self->xpos)) > (scrW-1))	/* move forward and.. */
		self->behaviour = NULL;		/* Die when offscreen */

	FindColl(self);

	if(self->behaviour != NULL){
		if(xdata[self->xpos+self->ypos*scrW]=='-')
			xdata[self->xpos+self->ypos*scrW]='=';
		else xdata[self->xpos+self->ypos*scrW]='-';
	}
}


/* Behaviour for enemy's shot */
/* Modes: 0 -- Just spawned, don't move
**        1 -- Okay, you can move now  */
void B_Theirshot(Actor* self){

	if(self->mode==0) self->mode = 1;	/* don't move, just spawned */
	else if((--(self->xpos)) < 1)		/* move backward and.. */
		self->behaviour = NULL;		/* Die when offscreen */
	FindColl(self);
	if(self->behaviour != NULL){
		if(xdata[self->xpos+self->ypos*scrW]=='-')
			xdata[self->xpos+self->ypos*scrW]='=';
		else xdata[self->xpos+self->ypos*scrW]='-';
	}
}


/* Rock. Just floats left.
** Modes:
** 0: dead
** 1: Small
** 2: Large
** 3: Larger/Tougher */
void B_Rock(Actor* self){

	assert(self->mode >0 && self->mode<4);

	/* Rocks are slow and only move every other frame */
	if(frame%2){
		if(--(self->xpos) < 0){
			score -=1;
			self->behaviour = NULL;
		}
	}

	FindColl(self);
	if(self->behaviour == B_Rock)
	switch(self->mode){
		case 3: xdata[self->xpos+self->ypos*scrW]='0'; break;
		case 2: xdata[self->xpos+self->ypos*scrW]='O'; break;
		case 1: xdata[self->xpos+self->ypos*scrW]='o'; break;
		default: self->behaviour = B_Explode;
	}
}


/* Drunk: Enemy with poor(random) AI
** Modes:
** 1: 1 hit left
** 2: 2 hits left
*/
void B_Drunk(Actor* self){

	int i =random()%7;

	/* Enemies only move every other frame */
	if(frame%2){
		if(self->xpos > scrW-10){
			--(self->xpos);
		}else
		if(i==1&&self->xpos>0) self->xpos--;
		else if(i==2&&self->xpos<scrW-2) self->xpos++;
		else if(i==3&&self->ypos>0) self->ypos--;
		else if(i==4&&self->ypos<scrH-2) self->ypos++;
		else if(i==5&&self->ypos>0 && self->xpos>0){self->xpos--;self->ypos--;}
		else if(i==6&&self->ypos<scrH-2&&self->xpos>0){
			self->xpos--;self->ypos++; }
		else registerActor(self->ypos, (self->xpos-1), 0, &B_Theirshot);
	}
	FindColl(self);

	if(self->xpos <= 0 || self->xpos > scrW || self->ypos <= 0
		|| self->ypos > scrH)self->behaviour = NULL;
	if(self->behaviour == B_Drunk)
	switch(self->mode){
		case 2: xdata[self->xpos+self->ypos*scrW]='{'; break;
		case 1: xdata[self->xpos+self->ypos*scrW]='['; break;
		default: assert(0);
	}
}

/* YRespawn:
** Placeholder for a respawning player.
** Mode is turns until respawn
** Mode 0: Attempt to spawn
** Respawning items blink on for two frames, then off for two frames
*/
void B_YRespawn(Actor* self){

	playerCount++;
	if(--(self->mode)<1){
		/* Don't respawn without a safe path */
		if(data[self->xpos + (self->ypos * scrW)]==' '
			/* Either the place to be checked is out of bounds
				Or the place to be checked is empty */
			&&(self->xpos>=scrW-1
				||data[(self->ypos)*scrW+self->xpos+1]==' ')
			&&(self->xpos>=scrW-2
				||data[(self->ypos)*scrW+self->xpos+2]==' ')
			&&(self->xpos>=scrW-3
				||data[(self->ypos)*scrW+self->xpos+3]==' ')
			&&(self->xpos>=scrW-4
				||data[(self->ypos)*scrW+self->xpos+4]==' ')
			&&(self->xpos>=scrW-5
				||data[(self->ypos)*scrW+self->xpos+5]==' ')
			&&(self->xpos<1
				||data[(self->ypos)*scrW+self->xpos-1]==' ')
			&&(self->ypos>=scrH-1
				||data[(self->ypos+1)*scrW+self->xpos]==' ')
			&&(self->ypos<1
				||data[(self->ypos-1)*scrW+self->xpos]==' ')
		)
			self->behaviour = B_Player;
		else /* If respawn area unsafe, wait */
			self->mode = random()%4+1;
		/* The random() wait time makes the blinking
		look iffy, but works around a bug that had caused
		two adjacent B_YRespawns to each keep the other
		from respawning */
	}

	/* Blink */
	if(self->mode%4==2||self->mode%4==3)
		xdata[self->xpos+self->ypos*scrW] = '>';
}


/* Bonus item:
** Modes:
** 1: extra wingman
** 2: small score bonus
** 3: large score bonus
** 4: Fire bonus
*/
void B_Bonus(Actor* self){

	char c = ' ';
	/* Bonus only moves every other frame */
	if(frame%2)
		if(--(self->xpos) < 0){
			self->behaviour = NULL;
			score-=1;
		}

	FindColl(self);
	if(self->behaviour == B_Bonus){
		switch(self->mode){
		case 1: c='L'; break;
		case 2: c='S'; break;
		case 3: c='B'; break;
		case 4: c='F'; break;
		default: assert(0);
		}
		xdata[self->xpos+self->ypos*scrW] = c;
	}
}

/*** Collision detection ***/


/* Find Collisions between an Actor and other Actors */
void FindColl(Actor * self){
	int i=0;
	if(self==NULL) assert(0);
	while(i<maxActors && self->behaviour != NULL){
		if(ActorList[i].xpos == self->xpos
			&& ActorList[i].ypos == self->ypos
			&& ActorList[i].behaviour != NULL
			&& &(ActorList[i]) != self){
			Collide(self, &(ActorList[i]));
		}
		++i;
	}
}


/* Collide(): One big function that handles results of collisions.
** About half the results will swap the parameters and call Collide again.
**
** This collision detection *sucks*:
** Shots commonly pass through B_Player, B_Bonus objects
*/
void Collide (Actor* a, Actor* b){
	if(a==NULL)assert(0);if(b==NULL)assert(0);
	/* Switches require something that breaks down to ints. Damnit. */
	if(a->behaviour == B_Explode || b->behaviour == B_Explode
	|| a->behaviour == B_YRespawn || b->behaviour == B_YRespawn)
		/* explosions and respawns do nothing */;
	else
	if(a->behaviour == B_Player){
		if(b->behaviour == B_Player){
		/* If you manage to collide with yourself */
			a->behaviour = B_Explode;
			b->behaviour = B_Explode;
		}else if(b->behaviour == B_Yourshot||b->behaviour==B_Theirshot){
		/* You don't run into shots, shots run into you */
			Collide(b,a);
		}else if(b->behaviour == B_Rock){
			Collide(b,a); /* Same for rocks */
		}else if(b->behaviour == B_Drunk){
			Collide(b,a);
		}else if(b->behaviour == B_Bonus){
			switch(b->mode){
			case 1: b->mode = 20; b->behaviour = B_YRespawn;break;
			case 2: score+=15; b->behaviour = NULL; break;
			case 3: score+=40; b->behaviour = NULL; break;
			case 4: b->behaviour = NULL;
				registerActor(b->ypos,b->xpos+2,1,&B_Yourshot);
				registerActor(b->ypos,b->xpos+1,1,&B_Yourshot);
				registerActor(b->ypos-1,b->xpos+1,1,&B_Yourshot);
				registerActor(b->ypos+1,b->xpos+1,1,&B_Yourshot);
				registerActor(b->ypos-2,b->xpos,1,&B_Yourshot);
				registerActor(b->ypos+2,b->xpos,1,&B_Yourshot);
				registerActor(b->ypos-3,b->xpos,1,&B_Yourshot);
				registerActor(b->ypos+3,b->xpos,1,&B_Yourshot);
				registerActor(b->ypos-3,b->xpos-1,1,&B_Yourshot);
				registerActor(b->ypos+3,b->xpos-1,1,&B_Yourshot);

				break;
			default: assert(0);
			}
		}else assert(0);
		
	} else if(a->behaviour == B_Yourshot){
		if(b->behaviour ==  B_Player){ 
			/* No blowing yourself up, please */
			/* Actually, the bug ridden collision detection
			   prevents an easy way of allowing this which
			   would not kill you if you fire and move right */
			;
		}else if (b->behaviour==B_Yourshot||b->behaviour==B_Theirshot){
			; /* Do nothing */
		}else if(b->behaviour == B_Rock){
			score++;
			if(--(b->mode) < 1){
				b->behaviour = B_Explode;
			}
			a->behaviour = NULL;
		}else if(b->behaviour == B_Drunk){
			a->behaviour = NULL;
			score+=3;
			if(--(b->mode) <1){
				score+=2;
				b->behaviour = B_Explode;
			}
		}else if(b->behaviour == B_Bonus){
			a->behaviour=NULL;
			b->behaviour=B_Explode;
			score-=2;
		}else assert(0);
	} else if(a->behaviour == B_Theirshot){
		if(b->behaviour ==  B_Player){ /* Ow. */
			a->behaviour = B_Explode;
			b->behaviour = NULL;
		}else if (b->behaviour==B_Yourshot||b->behaviour==B_Theirshot){
			; /* Do nothing */
		}else if(b->behaviour == B_Rock){
			a->behaviour = NULL;
			if(--(b->mode) < 1){
				b->behaviour = B_Explode;
			}
			a->behaviour = NULL;
		}else if(b->behaviour == B_Drunk){
			a->behaviour = NULL;
			if(--(b->mode) <1){
				b->behaviour = B_Explode;
			}
		}else if(b->behaviour == B_Bonus){
			;
		}else assert(0);
	} else if(a->behaviour == B_Rock){
		if(b->behaviour ==  B_Player){
			b->behaviour = B_Explode;
			if(--a->mode <1) a->behaviour = NULL;
		}else if (b->behaviour == B_Yourshot||b->behaviour==B_Theirshot){
			Collide(b,a);
		}else if(b->behaviour == B_Rock){
			if(a->mode > b->mode) b->behaviour = NULL;
			else a->behaviour = NULL;
		}else if(b->behaviour == B_Drunk){
			registerActor(b->ypos, b->xpos,1, B_Explode);
			if(--(b->mode) < 1){ b->behaviour = NULL;}
			if(--(a->mode) < 1){ a->behaviour = NULL;}
		}else if(b->behaviour == B_Bonus){
			if(--(a->mode) < 1){ a->behaviour = NULL;}
			b->behaviour=B_Explode;
		}else assert(0);
	} else if(a->behaviour == B_Drunk){	
		if(b->behaviour == B_Player){
			a->behaviour = B_Explode;
			b->behaviour = B_Explode;
		}else if(b->behaviour == B_Yourshot||b->behaviour==B_Theirshot){
			Collide(b,a);
		}else if(b->behaviour == B_Rock){
			Collide(b,a);
		}else if(b->behaviour == B_Drunk){
			a->behaviour = B_Explode;
			b->behaviour = B_Explode;
		}else if(b->behaviour == B_Bonus){
			b->behaviour = NULL;
		}else assert(0);
	}else if(a->behaviour == B_Bonus){
		if(b->behaviour==B_Bonus)
			a->behaviour=NULL;
		else Collide(b,a);
	}else assert(0);

}

