#ifndef CSHOOT_ACTOR_H
#define CSHOOT_ACTOR_H

/* Actor struct:
** Everything that moves on the screen is an Actor.
** Each Actor has one of several Behaviours that determines how it moves
** and what is displayed on the screen to represent it.
** An Actor is killed by setting its behaviour to NULL. */
struct Actor {
	int ypos,	/* Vertical position */
	    xpos,	/* Horizontal position */
	    mode;	/* Extra variable for behaviour's (optional) use;
			** often used to represent status or hit points*/
	void (*behaviour)(struct Actor*);       /* Pointer-to-function */
}; typedef struct Actor Actor;



/* registerActor() -- Add an Actor to the ActorList
** The last argument is the address of a Behaviour function
** which returns void and takes an Actor* as its argument */
Actor* registerActor(int,int,int, void(*)(Actor*));
  
/* fixActorList() -- Remove killed Actors from ActorList */
void fixActorList(void); 

#endif /* CSHOOT_ACTOR_H */

