/* cshoot.c
** A text-based shooter
** Initial release by David Turover, May 2002
** Color added by Ian Wilson, December 2002
** Developer's release January 4, 2003. 
** This code is released into the public domain.
** compile with -lcurses:
** gcc cshoot.c -o cshoot -lcurses
*/

/* How to play:
 ** Escape: Quit.
 ** Space Bar: Fire
 ** use Arrows to move around or:
 ** h: left
 ** j: down
 ** k: up
 ** l: right
 ** Pick up special bonuses:
 ** L: Laddie wingman
 ** S: Score bonus
 ** B: score Bonus
 ** F: Fire wide shot
 ** Avoid the rocks: o O 0  and enemies: { [ and their shots
 ** Control-c is the panic button if Escape doesn't work
 */


#include <stdlib.h>
#include <curses.h>
#include <signal.h>
#include "include/mainlib.h"
#include "include/actor.h"
#include "include/behaviour.h"
#include "include/ncurslib.h"


int scrH=18,		/* The Screen Height */
    scrW=78,		/* The Screen Width  */
    input,		/* Value of keypress. Global to effect many targets */
    *data = NULL,	/* Data of last complete frame */
    *xdata = NULL,	/* Data of frame currently being constructed */
    *clrdata = NULL,	/* Empty data set */
    playerCount = 0,	/* Number of player-controlled objects on screen */
    score = 0;		/* Your score */
    
long frame = 0;		/* Frame number. This is used by some behaviours	   			** so as not to run every frame; Currently, only
			** shots and the player move every frame */

Actor * ActorList;		/* Memory for storing the Actors */
const int maxActors = 256;	/* Max. number of actors in game at one time */

int main(int argc, char **argv)
{
	int i;
	/*
	printf("Starting Program...\n");
	*/
        signal(SIGINT, finish);	     /* run finish() on sigINT (^C)  */
	srandom(time((time_t)NULL)); /* Start the random number generator */
	/* Start the Ncurse Terminal */
	/*
        printf("Starting Ncurse Terminal...\n");
	*/

        initNcurse();

/* Malloc the game data */
	data = (int*)malloc(scrW * scrH * sizeof(int));
	xdata = (int*)malloc(scrW * scrH * sizeof(int));
	clrdata = (int*)malloc(scrW * scrH * sizeof(int));
	ActorList = (Actor*)malloc(maxActors * sizeof(Actor));

	/* Check for malloc failure */
	if(data == NULL || xdata == NULL 
		|| clrdata == NULL || ActorList == NULL){
		printf("ERROR: Unable to allocate memory.\n");
		finish(0);
	}

	/* Clear data */
	for(i=0;i<maxActors;i++) ActorList[i].behaviour = NULL;
	for(i=0;i<scrW*scrH;i++) clrdata[i] = ' ';
	memcpy(data, clrdata, scrW*scrH*sizeof(int));
	memcpy(xdata, clrdata, scrW*scrH*sizeof(int));

	/* Draw a Border */
	drawBorder();


	/* Add you to the screen */
	registerActor(scrH/2,scrW/20+2,1,&B_Player);
		
	/* Main program Loop */
	doMainLoop();
		
	finish(0); /* Finish nicely in case doMainLoop returns */

	return EXIT_SUCCESS; /* This prevents a compiler warning */
	/* finish() will exit() before we reach here. */
}
