#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>	/* for usleep */
#include <curses.h>
#include "include/mainlib.h"
#include "include/behaviour.h"
#include "include/ncurslib.h"
#include "include/actor.h"

extern int color;
extern int frame;
extern int playerCount;
extern int scrW;
extern int scrH;
extern int *data;
extern int *xdata;
extern int * clrdata;
extern int score;
extern Actor * ActorList;
extern int maxActors;
extern int input;

/* finish(): clean some things up at the end of the program */
void finish(int sig)
{
	endwin();	/* Keeps you from having to reset your terminal */
	if (data != NULL) free(data);
	if (xdata != NULL) free(xdata);
	if (clrdata != NULL) free(clrdata);
	if (ActorList != NULL) free(ActorList);
	exit(sig);
}



void doMainLoop(void)
{
	int i;	/* Generic Iterator */
	int difficulty = 0;

	do
	{
		++frame;		/* Increase frame number */
		playerCount = 0;	/* Reset playerCount */
		input = getch();	/* Get your input */

		/* Clear the new-data buffer */
		memcpy(xdata, clrdata, scrW*scrH*sizeof(int));

		/* Run each Actor's behaviour */
		for(i=0;i<maxActors; i++){
			if (ActorList[i].behaviour != NULL)
			ActorList[i].behaviour(&(ActorList[i]));
		}
		fixActorList();

		/* Randomly add a Rock every so often */
		if(random()%3 == 0)
			registerActor(random()%scrH,scrW-1,random()%3+1,&B_Rock);

		/* Add a Drunk every few screens */
		if(random()%9 ==0)
			registerActor(random()%(scrH-6)+3,scrW-1,2,&B_Drunk);

		/* Add a bonus every few screens */
		if(random()%17 ==0){
			registerActor(random()%scrH,scrW-1,random()%4+1,&B_Bonus);
		}

		/* copy new data over old data */
		/* The old data had been needed by behaviour()s */
		memcpy(data, xdata, scrW*scrH*sizeof(int));

		/* draw new data to screen and refresh */
		for(i=0; i<scrW*scrH; i++){
			if(color) addColor(data[i]);
			mvaddch(1+i/scrW,1+i%scrW,data[i]);
			if(color) removeColor(data[i]);
		}
		printScore(); 
		refresh();

		/* Handle death */
		/* If the player's score is too low,
		** there is no respawn and the game ends;
		** If the score is high enoungh, the player
		** loses a few points and respawns  */
		if(playerCount ==0 && score > 100){
			score-=75;	/* Inflict a penalty */
			registerActor(scrH/2,scrW/20+2,16,&B_YRespawn);
			playerCount=1;
		}

		/* Quit on Escape key or 'Q' command */
		/* This is not in B_Player, where other input is processed,
	 	*  because B_YRespawn's behaviour will raise playerCount */
		if(input == 'Q' || input == KEY_ESC)
			 playerCount = 0;

		/* Turn color on/off */
		if(input == 'c' || input == 'C'){
			color = (!color) && has_colors();
			drawBorder();
		}

		/* Change the difficulty value */
		/* At higher scores, the game is faster */
		if(score > 100){
			difficulty = (score-100)/20;
			if(difficulty > 52) difficulty = 52;
		} else difficulty = 0;
	  
		/* Sleep */
		usleep(1024*(64-difficulty));
	}while(playerCount > 0);
}
